import 'dart:async';
import 'package:graphql_flutter/graphql_flutter.dart';
import 'package:http/http.dart' as Http;

typedef HttpResponseHandler = void Function(Response?, Http.Response);

class GraphQLInterceptHttpCodeLink extends Link {
  HttpResponseHandler httpResponseHandler;
  late StreamController<Response> controller;
  late Request gqlRequest;
  NextLink? forward;

  GraphQLInterceptHttpCodeLink({required this.httpResponseHandler});

  @override
  Stream<Response> request(Request request, [NextLink? forward]) {
    gqlRequest = request;
    this.forward = forward;
    controller = StreamController<Response>(onListen: onListen);

    return controller.stream;
  }

  void onListen() async {
    await controller.addStream(forward!(gqlRequest).handleError((error) {
      if(error != null && error is HttpLinkServerException){
        Http.Response httpResponse = error.response;
        if (httpResponse != null) {
          switch (httpResponse.statusCode) {
            case 304:
            case 400:
            case 401:
            case 403:
            case 404:
            case 500:
            case 501:
            case 502:
            case 503:
            case 306:
              httpResponseHandler(null, httpResponse);
              break;
            case 402:
              String? message;
              String? errorMessage = httpResponse.body != null ? httpResponse.body : '';
              if (!isStringEmpty(errorMessage)) {
                List msgList = errorMessage.split(':');
                if (msgList.length >= 2) {
                  message = msgList[1].toString().trim();
                }
              }

              Http.Response addStreamedResponse = Http.Response(
                  httpResponse.body, httpResponse.statusCode,
                  reasonPhrase: isStringEmpty(message)
                      ? httpResponse.reasonPhrase
                      : message);
              httpResponseHandler(null, addStreamedResponse);
              break;
          }
        }
      }
      throw error;
    }
    ));

    await controller.close();
  }

  static bool isStringEmpty(String? value) {
    return value == null || value.length <= 0;
  }
}


